/*****************************************************************************/
/*	asmount - AfterStep Mounter					     */
/*	Version 1.0.0						             */
/*	By Erik Johnsson						     */
/*                                                                           */
/*	zaphod@linux.nu 						     */
/*	http://www.cs.hope.edu/~malda/					     */
/*      based on AfterStep Modem                  			     */
/*									     */
/*****************************************************************************/

#define DEFAULT_XPM 1

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>

/* XPM struct and icons ******************************************************/

typedef struct _XpmIcon {
	Pixmap pixmap;
	Pixmap mask;
	XpmAttributes attributes;
} XpmIcon;

XpmIcon Mounted, NotMounted, Current;

#ifdef DEFAULT_XPM
	#include "XPM/mounted_def.xpm"
	#include "XPM/notmounted_def.xpm"
#else
	#include "XPM/mounted.xpm"
	#include "XPM/notmounted.xpm"
#endif

/* Functions *****************************************************************/

void	CreateWindow(void);
void	MainLoop();
void	GetXPM(void);
int	FlushExpose(Window w);
void	RedrawWindow( XpmIcon *v);
Pixel	GetColor(char *name);
int     mountcheck(void);

/* Global stuff **************************************************************/

#define TRUE 1;
#define FALSE 0;
#define DEFINTERVAL 1;
#define TICSPERSEC 6;

Display	*Disp;	 
Window	Root;
Window	Iconwin;
Window	Win;
char	*Geometry= 0;
int	Shape = 1;
GC	WinGC;
int     IsMounted = FALSE;
int     Interval = DEFINTERVAL;
int     TicsperInt;
char    *device = "/dev/cdrom";
char    *mountpoint = "/cdrom";
int     readwrite = FALSE;

/*****************************************************************************/

void Help()
{       
	printf("asmount - Version 1.0.0\n");
	printf("usage:  asmount [-options ...] \n");
	printf("options:\n");
	printf("        -u <sec>         update frequency (default 1 sec)\n");
	printf("        -d <device>   	 mount device(e.x /dev/cdrom\n");
	printf("        -m <mount point> mount point (e.x /cdrom)\n");
	printf("        -w               mount the device read/write\n");
	printf("\n");       
	exit(1);
}

/*****************************************************************************/

void ParseCmdLine(int argc, char *argv[])
{

	char *Argument;
	int i;       
	
	for(i = 1; i < argc; i++) 
	{
		Argument = argv[i];
		
		if (Argument[0] == '-') 
		{
			switch(Argument[1]) 
			{
			 case 'u':
				if(++i >= argc) Help();				
				Interval = atoi(argv[i]);
				if(!Interval)
				{
					fprintf(stderr, "asmount: invalid update frequency specified, using default\n");
					Interval = DEFINTERVAL;  /* Use default */
				}
				continue;
			case 'd':
			        if(++i >= argc) Help();

			        device = argv[i];
				continue;
			case 'm':
			        if(++i >= argc) Help();
			 
			        mountpoint = argv[i];
				continue;
			case 'w':
			        readwrite = TRUE;
				continue;
			 default:
				Help();
			}
		}
		else
			Help();
	}
}

/*****************************************************************************/

int main(int argc,char *argv[])
{       	
     	ParseCmdLine(argc, argv);      

	TicsperInt = Interval * TICSPERSEC;
	CreateWindow();
        XSetCommand(Disp,Win,argv,argc); 
	MainLoop();
	return 0;
}

/*****************************************************************************/

void CreateWindow(void)
{
	int i;
	unsigned int borderwidth ;
	char *display_name = NULL; 
	char *wname = "asmount";
	XGCValues gcv;
	unsigned long gcm;
	XTextProperty name;
	Pixel back_pix, fore_pix, third_pix;
	Pixmap pixmask;
	int screen;	
	int x_fd;
	int d_depth;
	int ScreenWidth, ScreenHeight;
	XSizeHints SizeHints;
	XWMHints WmHints;
        XClassHint classHint;   
	int check;

	check = mountcheck();
	if (check < 0)
	  {
	    fprintf(stderr, "asmount: /proc filesystem must be enabled in the kernel for asmount to work.\n");
	    exit(-1);
	  }
	
	/* Open display */
	if (!(Disp = XOpenDisplay(display_name)))  
	{ 
		fprintf(stderr,"asmount: can't open display %s\n", 
			XDisplayName(display_name)); 
		exit (1); 
	} 
	
	screen = DefaultScreen(Disp);
	Root = RootWindow(Disp, screen);
	d_depth = DefaultDepth(Disp, screen);
	x_fd = XConnectionNumber(Disp);	
	ScreenHeight = DisplayHeight(Disp,screen);
	ScreenWidth = DisplayWidth(Disp,screen);
	       
	GetXPM();
		
	SizeHints.flags= USSize|USPosition;
	SizeHints.x = 0;
	SizeHints.y = 0;	
	back_pix = GetColor("black");
	fore_pix = GetColor("black");
 	third_pix = GetColor("white");	
	XWMGeometry(Disp, screen, Geometry, NULL, (borderwidth =1), &SizeHints,
		    &SizeHints.x,&SizeHints.y,&SizeHints.width,
		    &SizeHints.height, &i); 	
	SizeHints.width = NotMounted.attributes.width;
	SizeHints.height= NotMounted.attributes.height;	
	Win = XCreateSimpleWindow(Disp,Root,SizeHints.x,SizeHints.y,
				  SizeHints.width,SizeHints.height,
				  borderwidth,fore_pix,back_pix); 
	Iconwin = XCreateSimpleWindow(Disp,Win,SizeHints.x,SizeHints.y,
				      SizeHints.width,SizeHints.height,
				      borderwidth,fore_pix,back_pix);	     

	classHint.res_name =  "asmount";
        classHint.res_class = "ASMount";
        XSetClassHint(Disp, Win, &classHint);     

	XSetWMNormalHints(Disp, Win, &SizeHints);
	XSelectInput(Disp, Win, (ExposureMask | ButtonPressMask | 
				 StructureNotifyMask));
	XSelectInput(Disp, Iconwin, (ExposureMask | ButtonPressMask | 
				     StructureNotifyMask));
	if (XStringListToTextProperty(&wname, 1, &name) ==0) 
	{
		fprintf(stderr, "asmount: can't allocate window name\n");
		exit(-1);
	}
	XSetWMName(Disp, Win, &name);
	
	/* Create WinGC */
	gcm = GCForeground|GCBackground|GCGraphicsExposures;
	gcv.foreground = GetColor("white");
	gcv.background = GetColor("black");
	gcv.graphics_exposures = 0;
	WinGC = XCreateGC(Disp, Root, gcm, &gcv);  


        WmHints.initial_state = NormalState;

        WmHints.window_group = Win;
        WmHints.flags = StateHint | IconWindowHint | IconPositionHint
          | WindowGroupHint;

	WmHints.icon_window = Iconwin;
	WmHints.icon_x = SizeHints.x;
	WmHints.icon_y = SizeHints.y;
	XSetWMHints(Disp, Win, &WmHints); 	
	XMapWindow(Disp,Win);
	RedrawWindow(&Current);
}

/****************************************************************************/

void MainLoop()
{
	XEvent Event;            
	XpmIcon Prev;
	int tics=0,cp;
	char *CommandLine=malloc(128);

	Prev = Current;

	/* Main loop */
	while(1)
	{	
	  tics++;
		if (Prev.pixmap != Current.pixmap)
		  {
			RedrawWindow(&Current);
		  }

		Prev = Current;
		if (IsMounted==0)
		  Current=NotMounted;
		else
		  Current=Mounted;
		/* Check events */
		while (XPending(Disp))
		  {
			XNextEvent(Disp,&Event);
		        
			switch(Event.type)
			  {		     
			  case Expose:		/* Redraw window */
				if(Event.xexpose.count == 0)
					RedrawWindow(&Current); 
				break;		     
			  case ButtonPress:	/* Mouseclick */
			        if (IsMounted==0)
				  {
				    sprintf(CommandLine,"mount %s %s",device,mountpoint);
				    if (!readwrite)
				      sprintf(CommandLine,"%s -r",CommandLine);
				    system (CommandLine);
				  }
				else
				  {
				    sprintf(CommandLine,"umount %s",mountpoint);
				    system (CommandLine);
				  }
				break;
			  case DestroyNotify:	/* Destroy window */
				XFreeGC(Disp, WinGC);
				XDestroyWindow(Disp, Win);
				XDestroyWindow(Disp, Iconwin);
				XCloseDisplay(Disp);				
				exit(0); 
				break;			
			  }
		  }
		if (tics == TicsperInt)
		  {
		    tics = 0;
		    IsMounted=mountcheck();
		  }

		XFlush(Disp);
		cp = 1000000 / TICSPERSEC;
		usleep(cp);
	}
}

/****************************************************************************/

void GetXPM(void)
{
	XWindowAttributes Attributes;
	int Ret;

	XGetWindowAttributes(Disp,Root,&Attributes);		

	NotMounted.attributes.valuemask |= (XpmReturnPixels|XpmReturnExtensions);
	Ret = XpmCreatePixmapFromData(Disp, Root, notmounted_xpm,
				      &NotMounted.pixmap, &NotMounted.mask,
				      &NotMounted.attributes);
	if(Ret != XpmSuccess)
	{
		fprintf(stderr, "asmount: not enough free color cells\n");
		exit(1);
	}
	Mounted.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
	Ret = XpmCreatePixmapFromData(Disp, Root, mounted_xpm, &Mounted.pixmap, 
				      &Mounted.mask, &Mounted.attributes);
	Current = NotMounted;	  
}

/****************************************************************************/

int FlushExpose(Window w)
{
	XEvent dummy;
	int i=0;
	
	while (XCheckTypedWindowEvent (Disp, w, Expose, &dummy))i++;
	return i;
}

/****************************************************************************/

void RedrawWindow(XpmIcon *Icon)
{		
	FlushExpose(Win);
	FlushExpose(Iconwin);
	
	if(Shape==1) 
	{ 				
		XShapeCombineMask(Disp, Win, ShapeBounding, 0, 0, 
				  Icon->mask, ShapeSet);
		XShapeCombineMask(Disp, Iconwin, ShapeBounding, 0, 0, 
				  Icon->mask, ShapeSet);
	}

	XSetClipOrigin(Disp,WinGC,0,0);
	XSetClipMask(Disp,WinGC,Icon->mask);
	
	XCopyArea(Disp,Icon->pixmap,Win,WinGC,
		  0,0,Icon->attributes.width, Icon->attributes.height,0,0);
	XCopyArea(Disp,Icon->pixmap,Iconwin,WinGC,
		  0,0,Icon->attributes.width, Icon->attributes.height,0,0);

}

/****************************************************************************/

Pixel GetColor(char *ColorName)
{
	XColor Color;
	XWindowAttributes Attributes;
	
	XGetWindowAttributes(Disp,Root,&Attributes);
	Color.pixel = 0;
	
	if (!XParseColor (Disp, Attributes.colormap, ColorName, &Color)) 
		fprintf(stderr,"asmount: can't parse %s\n", ColorName);
	else if(!XAllocColor (Disp, Attributes.colormap, &Color)) 
		fprintf(stderr,"asmount: can't allocate %s\n", ColorName);       
	
	return Color.pixel;
}

int mountcheck(void)
{
        char *file,*tmp = malloc(256),*cp = malloc(256);
        int fd, cmp, ret, tmp2;

	tmp = cp;

        file = "/proc/mounts";
        fd = open (file, O_RDONLY);
        if (fd <= 0)
        {
                perror(file);
                return(-2);
        }        
	read(fd, tmp, 256);

	tmp2 = strstr(tmp,mountpoint);

        if (tmp2 == 0)
	    ret = 0;
	else
	    ret = 1;

        free(tmp);
        close(fd);
        return(ret);
}
